# encoding: utf-8
# cython: profile=False
from __future__ import division
import numpy as np
cimport numpy as np
cimport cython
cimport fox_goodwin

@cython.boundscheck(False)
@cython.wraparound(False) 
cpdef fox_goodwin(np.ndarray[np.float64_t,ndim=1] z, func_w, bint write_p=False):
    """
        solve transaction matrix 
    """
    cdef np.float64_t h, c1
    cdef np.int16_t nr, i
    cdef np.ndarray[np.float64_t,ndim=2] w_m1,w,m_p1,r,I,I2,aa,bb,cc
    h = z[1] - z[0]
    c1 = h ** 2 / 12.0

    w_m1 = c1 * func_w(z[0], 0)
    w = c1 * func_w(z[1], 1)

    nr = w.shape[0]

    r = np.zeros([nr, nr],dtype=np.float64)
    r_dict = {}
    I = np.eye(nr,dtype=np.float64)
    I2 = 2 * I

    for i in range(1, len(z) - 1):
        w_p1 = c1 * func_w(z[i + 1], i + 1)

        # performing a Numerov step
        aa = I2 + 10.0 * w
        bb = np.dot(I - w_m1, r)
        cc = I - w_p1
        
        r = np.linalg.solve((aa - bb), cc)

        # if the flag was set, the R-matrix is stored in the R-dictionary
        if write_p:
            r_dict[z[i]] = r

        # preparations for the next step
        w_m1 = w
        w = w_p1

    if write_p:
        return r_dict
    else:
        return r
