import pytest
import haslib as has
from haslib.science_basics import A, mel
from haslib.icc_2d_2 import iCC_2D_2 as icc2
from haslib.icc_2d import iCC_2D as icc
import numpy as np


def sort(e):
    ind = np.lexsort((e[:, 0], e[:, 1]))
    return e[ind, :]


@pytest.fixture
def old_args():
    a = 4*A
    Bi2Te3_111 = has.Crystal(a, a, 120)
    Bi2Te3_111.rotate_crystal(1, 1)
    g1 = Bi2Te3_111.b1[0:2]
    g2 = Bi2Te3_111.b2[0:2]
    args = {
            "a": a,
            "h": 0.7,
            "xi": 0.2/A,
            "D": 4*mel,
            "maxrad": 20,
            "Ei": 13 * mel,
            "theta_i": 22.2,
            "G_clo": 10,
            "G1": g1,
            "G2": g2,
            }
    return args


@pytest.fixture
def new_args(old_args):
    args = old_args.copy()
    return args


@pytest.fixture
def icc_args(new_args):
    args = new_args.copy()
    args["phon"] = [0, 0, 0]
    args["floq"] = 0
    args["AP"] = 1e-22
    args["Gclo"] = args["G_clo"]
    del args["G_clo"]
    return args


def test_cc(old_args, new_args):
    new = sort(has.CC_2D(**new_args))
    old = sort(has.CC_2D_old(**old_args))
    assert np.allclose(new, old)


def test_icc(icc_args):
    args = icc_args
    args["floq"] = 1
    new = sort(icc2(**args))
    old = sort(icc(**args))
    assert np.allclose(new, old)


def test_cc_icc(icc_args, new_args):
    new = sort(has.CC_2D(**new_args))
    old = sort(icc2(**icc_args)[:, [0, 1, 6, 5]])
    assert np.allclose(new, old)


def test_integrate(new_args, icc_args):
    new = sort(has.CC_2D(**new_args))
    wD = 10.0

    def f(x):
        old = icc2(**icc_args)[:, [0, 1, 6, 5]]
        ret = np.array([old[(old[:, 0] == 0) & (old[:, 1] == 0), 3]])
        ret *= 3*x[0]**2/wD**3
        return ret

    ospec = has.integrate_GL(0.0, wD, f, n=10)
    nspec = new[(new[:, 0] == 0) & (new[:, 1] == 0), 3]
    assert np.allclose(ospec, nspec)
