import pytest
from collections import namedtuple
import haslib as has
from haslib.science_basics import A, meV
import numpy as np


def sort(e):
    ind = np.lexsort((e[:, 0], e[:, 1]))
    return e[ind, :]


@pytest.fixture
def args():
    args = namedtuple("args", ["e", "phi", "theta", "pot"])
    a = 4*A
    Bi2Te3_111 = has.Crystal(a, a, 120)
    Bi2Te3_111.rotate_crystal(1, 1)
    args.e = np.array([1*meV])
    args.phi = np.array([12])
    args.theta = 22.2
    h = 0.07
    xi = 0.7/A
    D = 4*meV
    args.pot = has.Morse(Bi2Te3_111, h, xi, D)
    return args


def test_noargs(args):
    has.AzimuthalScan().run(args.e, args.phi, args.theta, args.pot)


def test_verbose(args, capsys):
    has.AzimuthalScan().run(args.e, args.phi, args.theta,
                            args.pot, cpus=0, verbose=True)
    output = capsys.readouterr().out
    assert "Information" in output


def test_zstep(args, capsys):
    has.AzimuthalScan().run(args.e, args.phi, args.theta,
                            args.pot, cpus=0, verbose=True, st_p_wl=180)
    output = capsys.readouterr().out
    assert "Number of Steps per minimum wavelength -->   180" in output
