import numpy as np
from haslib.science_basics import A, meV
import haslib.icc_2d as icc
import haslib.icc_2d_2 as icc2
import pytest


def sort(e):
    ind = np.lexsort((e[:, 0], e[:, 1], e[:,2]))
    return e[ind, :]

@pytest.fixture
def args():
    args = {
        "h": 0.8,
        "AP": 0.5,
        "a": 2*A,
        "D": 5*meV,
        "xi": 1/A,
        "Ei": meV,
        "theta_i": 22,
        "G1": np.array([1.0, 0])/A,
        "G2": np.array([0.3, 0.6])/A,
        "Gope": -1,
        "Gclo": 10,
        "phon": [0, 0, 1*meV],
        #"st_p_wl": 30,
        #"end_z": 7*A,
        #"sta_z": -2*A
    }
    return args


def test_icc_python(benchmark, args):
    benchmark(icc.iCC_2D, **args)


def test_icc_cython(benchmark, args):
    benchmark(icc2.iCC_2D_2, **args)


def test_same_result(args):
    r1 = sort(icc.iCC_2D(**args))
    r2 = sort(icc2.iCC_2D_2(**args))
    assert np.allclose(r1, r2)
